/****************************************************************************
** $Id: main.cpp  beta1   edited Dec 18 12:16 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include <qfile.h>
#include <qprocess.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qregexp.h>
#include <qapplication.h>
#include <stdlib.h>
#ifdef Q_WS_WIN
#include <windows.h>
#endif

static QString *qtDir = 0;
static QString *assistant = 0;
static bool internalQt = FALSE;
static bool enterpriseQt = FALSE;
static bool usVersion = FALSE;
static const char *app = 0;
static int processes = 0;
static int errors = 0;

static void message( const QString &msg )
{
    qWarning( msg );
}

class ProcessListener : public QObject {
    Q_OBJECT
public:
    ProcessListener( QProcess *pr ) : p( pr ) { processes++; }
    ~ProcessListener() {
	delete p;
	if ( --processes == 0 )
	    QApplication::exit();
    }

public slots:
    void processExited() {
	if( p->exitStatus()!=0 )
	    errors++;
	deleteLater();
    }

    void readyReadStdout() {
	if( p->canReadLineStdout() ) {
	    qDebug( p->readLineStdout() );
	}
   }

    void readyReadStderr() {
	if( p->canReadLineStderr() ) {
	    qDebug(  p->readLineStderr() );
	}
    }

private:
    QProcess *p;
};


static void execute( const QStringList &args )
{
    QProcess *p = new QProcess( args );
    ProcessListener *l = new ProcessListener( p );
    QObject::connect( p, SIGNAL( readyReadStdout() ), l, SLOT( readyReadStdout() ) );
    QObject::connect( p, SIGNAL( readyReadStderr() ), l, SLOT( readyReadStderr() ) );
    QObject::connect( p, SIGNAL( processExited() ), l, SLOT( processExited() ) );
    if( !p->start() ) {
	delete l;
	qDebug( "Failed to start process: '" + args.join( " " ) + "'" );
	errors++;
    }

}

static void addContentFile( const QString &name )
{
    if ( !assistant ) {
	assistant = new QString( "assistant" );
#ifdef Q_OS_MACX
	QStringList dirs = QStringList::split( ':', getenv( "PATH" ) );
	QStringList::const_iterator it;
	for ( it = dirs.begin(); it != dirs.end(); ++it ) {
	    QString a = *it + "/assistant.app/Contents/MacOS/assistant";
	    if ( QFile::exists( a ) ) {
		*assistant = a;
		break;
	    }
	}
#endif
    }
    QStringList lst;
    lst.append( *assistant );
    lst.append( "-addContentFile" );
    lst.append( QDir::convertSeparators( name ) );
    execute( lst );
}

static void mkDir( const QString &dir )
{
    QDir current;
    current.mkdir( dir );
}

static void copy( const QString &source, const QString &dest )
{
    QString s = QDir::convertSeparators( source );
    QString d = QDir::convertSeparators( dest );
#ifdef Q_OS_UNIX
    system( "cp " + QFile::encodeName( s ) + " " + QFile::encodeName( d ) );
    system( "chmod +w " + QFile::encodeName( d ) );
#else
    QT_WA(
    {
	if ( !CopyFileW( (TCHAR*) s.ucs2(), (TCHAR*) d.ucs2(), FALSE ) ) {
	    qDebug( "Failed to copy file: " + s );
	    errors++;
	}
	if ( !SetFileAttributesW( (TCHAR*) d.ucs2(), FILE_ATTRIBUTE_NORMAL ) ) {
	    qDebug( "Failed to set file attributes to normal" );
	    errors++;
	}
    }, {
	if ( !CopyFileA( QFile::encodeName( s ), QFile::encodeName( d ), FALSE ) ) {
	    qDebug( "Failed to copy file: " + s );
	    errors++;
	}
	if ( !SetFileAttributesA( QFile::encodeName( d ), FILE_ATTRIBUTE_NORMAL ) ) {
	    qDebug( "Failed to set file attributes to normal" );
	    errors++;
	}
    } );

#endif
}

static void symLink( const QString &source, const QString &dest )
{
#ifdef Q_OS_UNIX
    QString s = QDir::convertSeparators( source );
    QFileInfo info( s );
    s = info.absFilePath();
    QString d = QDir::convertSeparators( dest );
    system( "rm -f " + QFile::encodeName( d ) );
    system( "ln -s " + QFile::encodeName( s ) + " " + QFile::encodeName( d ) );
#else
    copy( source, dest );
#endif
}

static bool checkLicense()
{
    bool freeQSA = QFile::exists( "LICENSE.GPL" );
    bool freeQt = FALSE;

    if ( QFile::exists( *qtDir + "LICENSE.TROLL" ) ) {
	internalQt = TRUE;
	freeQt = TRUE;
    } else if ( QFile::exists( *qtDir + "LICENSE.GPL" ) ) {
	freeQt = TRUE;
    } else {
	QString home = QDir::homeDirPath();
	QString licenseFile = home + "/.qt-license";
	QFile f( licenseFile );
	if ( f.exists() ) {
	    if ( f.open( IO_ReadOnly ) ) {
		QString text = f.readAll();
		enterpriseQt = text.find( "\"qt-enterprise\"" ) >= 0;
		QRegExp re( "LicenseKey=([^\n]*)" );
		Q_ASSERT( re.isValid() );
		if ( re.search( text ) >= 0 ) {
		    QString key = re.cap( 1 );
		    if ( !key.isEmpty() ) {
			if ( QString( "3579BDFHbdfh" ).find( key[0] ) >= 0 )
			    usVersion = TRUE;
		    }
		}
	    }
	}
    }

    if ( freeQSA && freeQt )
	return TRUE;

    if ( freeQSA && !freeQt ) {
	message( "\nThis QSA Free Edition is licensed under the GNU General "
		 "Public License.\nNote that any derivatives of this software "
		 "must also be licensed under\nthe GNU GPL.\n\n"
		 "See http://www.trolltech.com/gpl for more information or\n"
		 "contact sales@trolltech.com for an Evaluation Edition.\n" );
	return TRUE;
    }
    if ( !enterpriseQt && !internalQt ) {
	message( "\nCan't find the required license information of a "
		 "Qt Enterprise Edition.\n" );
	return FALSE;
    }

    return TRUE;
}

int main( int argc, char **argv )
{
    QApplication qapp( argc, argv, FALSE );

    app = argv[0];
    qtDir = new QString( getenv( "QTDIR" ) );
    if ( qtDir->isEmpty() ) {
	qWarning( "%s: QTDIR not set", app );
	return 1;
    }
    *qtDir += "/";

    if ( !checkLicense() )
	return 1;

    // install appropriate license file
    if ( enterpriseQt )
	copy( usVersion ? ".LICENSE-US" : ".LICENSE", "LICENSE" );

    // install qmake feature file
    mkDir( *qtDir + "mkspecs/features" );
    copy( "src/qsa/qsa.prf", *qtDir + "mkspecs/features/qsa.prf" );

    // install header files
    QString headers[] = { "qsaglobal.h", "qsobjectfactory.h",
			  "qswrapperfactory.h", "qseditor.h",
			  "qsproject.h", "qsinterpreter.h" };
    for ( uint i = 0; i < sizeof(headers)/sizeof(QString); i++ )
	symLink( "src/qsa/" + headers[i], *qtDir + "include/" + headers[i] );

    // install documentation
    QString html = "doc/qsa/html/";
    if ( !QFile::exists( html ) )
	html = "doc/html/";
    addContentFile( html + "qsa.xml" );
    addContentFile( html + "qt-script-for-applications.xml" );
    addContentFile( html + "language.xml" );
    addContentFile( html + "library.xml" );
    addContentFile( html + "qsad.xml" );

    int retVal = processes > 0 ? qapp.exec() : 0;
    if( retVal || errors ) {
	qDebug( "\nThere was an error during configure!" );
    } else {
	qDebug( "\n"
		"Configuration completed successfully\n"
		"Run your make tool to build QSA" );
    }

    return retVal;
}

#include "main.moc"
