/****************************************************************************
** $Id: qsmath_object.cpp  beta1   edited Dec 10 14:29 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsmath_object.h"
#include "qstypes.h"
#include "qsoperations.h"
#include <math.h>
#include <stdlib.h>

using namespace QS;

QSMathClass::QSMathClass( QSClass *b )
    : QSClass( b )
{
    int a = AttributePublic|AttributeStatic;
    addStaticVariableMember( "E", QSNumber( ::exp( 1.0 ) ), a );
    addStaticVariableMember( "LN2", QSNumber( ::log( 2.0 ) ), a );
    addStaticVariableMember( "LN10", QSNumber( ::log( 10.0 ) ), a );
    addStaticVariableMember( "LOG2E", QSNumber( 1.0/::log( 2.0 ) ), a );
    addStaticVariableMember( "LOG10E", QSNumber( 1.0/::log( 10.0 ) ), a );
    addStaticVariableMember( "PI", QSNumber( 2.0 * ::asin( 1.0 ) ), a );
    addStaticVariableMember( "SQRT1_2", QSNumber( ::sqrt( 0.5 ) ), a );
    addStaticVariableMember( "SQRT2", QSNumber( ::sqrt( 2.0 ) ), a );

    // static functions
    addMember( "abs", QSMember( &abs, AttributeStatic ) );
    addMember( "acos", QSMember( &acos, AttributeStatic ) );
    addMember( "asin", QSMember( &asin, AttributeStatic ) );
    addMember( "atan", QSMember( &atan, AttributeStatic ) );
    addMember( "atan2", QSMember( &atan2, AttributeStatic ) );
    addMember( "ceil", QSMember( &ceil, AttributeStatic ) );
    addMember( "cos", QSMember( &cos, AttributeStatic ) );
    addMember( "exp", QSMember( &exp, AttributeStatic ) );
    addMember( "floor", QSMember( &floor, AttributeStatic ) );
    addMember( "log", QSMember( &log, AttributeStatic ) );
    addMember( "max", QSMember( &max, AttributeStatic ) );
    addMember( "min", QSMember( &min, AttributeStatic ) );
    addMember( "pow", QSMember( &pow, AttributeStatic ) );
    addMember( "random", QSMember( &random, AttributeStatic ) );
    addMember( "round", QSMember( &round, AttributeStatic ) );
    addMember( "sin", QSMember( &sin, AttributeStatic ) );
    addMember( "sqrt", QSMember( &sqrt, AttributeStatic ) );
    addMember( "tan", QSMember( &tan, AttributeStatic ) );
}

QSMathClass::~QSMathClass() {

}

QSObject QSMathClass::abs( const QSList &args )
{
    double arg = args[0].toNumber();
    return QSNumber( arg < 0 ? (-arg) : arg );
}

QSObject QSMathClass::acos( const QSList &args )
{
    return QSNumber( ::acos( args[0].toNumber() ) );
}

QSObject QSMathClass::asin( const QSList &args )
{
    return QSNumber( ::sin( args[0].toNumber() ) );
}

QSObject QSMathClass::atan( const QSList &args )
{
    return QSNumber( ::atan( args[0].toNumber() ) );
}

QSObject QSMathClass::atan2( const QSList &args )
{
    return QSNumber( ::atan2( args[0].toNumber(), args[1].toNumber() ) );
}

QSObject QSMathClass::ceil( const QSList &args )
{
    return QSNumber( ::ceil( args[0].toNumber() ) );
}

QSObject QSMathClass::cos( const QSList &args )
{
    return QSNumber( ::cos( args[0].toNumber() ) );
}

QSObject QSMathClass::exp( const QSList &args )
{
    return QSNumber( ::exp( args[0].toNumber() ) );
}

QSObject QSMathClass::floor( const QSList &args )
{
    return QSNumber( ::floor( args[0].toNumber() ) );
}

QSObject QSMathClass::log( const QSList &args )
{
    return QSNumber( ::log( args[0].toNumber() ) );
}

QSObject QSMathClass::max( const QSList &args )
{
    double arg0 = args[0].toNumber();
    double arg1 = args[1].toNumber();
    return QSNumber(  QMAX( arg0, arg1 ) );
}

QSObject QSMathClass::min( const QSList &args )
{
    double arg0 = args[0].toNumber();
    double arg1 = args[1].toNumber();
    return QSNumber(  QMIN( arg0, arg1 ) );
}

QSObject QSMathClass::pow( const QSList &args )
{
    double arg0 = args[0].toNumber();
    double arg1 = args[1].toNumber();
    return QSNumber( ::pow( arg0, arg1 ) );
}

QSObject QSMathClass::random( const QSList & )
{
    return QSNumber( ::rand() / RAND_MAX );
}

QSObject QSMathClass::round( const QSList &args )
{
    double arg = args[0].toNumber();
    double result;
    if ( isNaN( arg ) )
	result = arg;
    else if ( isInf( arg ) || isInf( -arg ) )
	result = arg;
    else if ( arg == -0.5 )
	result = 0;
    else
	result = qRound( arg );

    return QSNumber( result );
}

QSObject QSMathClass::sin( const QSList &args )
{
    return QSNumber( ::sin( args[0].toNumber() ) );
}

QSObject QSMathClass::sqrt( const QSList &args )
{
    return QSNumber( ::sqrt( args[0].toNumber() ) );
}

QSObject QSMathClass::tan( const QSList &args )
{
    return QSNumber( ::tan( args[0].toNumber() ) );
}
