/****************************************************************************
** $Id: quickcolorobject.cpp  beta1   edited Dec 10 14:22 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickcolorobject.h"
#include <qcolor.h>
#include <assert.h>

class ColorShared : public QSShared {
public:
    ColorShared( const QColor &c ) : color( c ) { }
    QColor color;
};

enum { Red, Green, Blue, Name, Rgb };

QSColorClass::QSColorClass( QSClass *b )
  : QSClass( b )
{
    // custom properties
    addMember( "red", QSMember( QSMember::Custom, Red, AttributeNone ) );
    addMember( "green", QSMember( QSMember::Custom, Green, AttributeNone ) );
    addMember( "blue", QSMember( QSMember::Custom, Blue, AttributeNone ) );
    addMember( "name", QSMember( QSMember::Custom, Name, AttributeNone ) );
    addMember( "rgb", QSMember( QSMember::Custom, Rgb, AttributeNone ) );

    // member function
    addMember( "setRgb", QSMember( &setRgb ) );
}

QColor *QSColorClass::color( const QSObject *obj ) const
{
    assert( obj->objectType() == this );
    return &((ColorShared*)obj->shVal())->color;
}

QSObject QSColorClass::fetchValue( const QSObject *objPtr,
				   const QSMember &mem ) const
{
    if ( mem.type() != QSMember::Custom )
	return QSClass::fetchValue( objPtr, mem );
    QColor *col = color( objPtr );
    switch ( mem.idx ) {
    case Red:
	return QSNumber( col->red() );
    case Green:
	return QSNumber( col->green() );
    case Blue:
	return QSNumber( col->blue() );
    case Rgb:
	return QSNumber( col->rgb() );
    case Name:
	return QSString( col->name() );
    default:
	qFatal( "QSColorClass::fetchValue: unhandled case" );
	return QSUndefined();
    }
}

void QSColorClass::write( QSObject *objPtr, const QSMember &mem,
			  const QSObject &val ) const
{
    if ( mem.type() == QSMember::Custom ) {
	QColor *col = color( objPtr );
	if ( mem.idx == Name ) {
	    col->setNamedColor( val.toString() );
	} else if ( mem.idx == Rgb ) {
	    col->setRgb( val.toUInt32() );
	} else {
	    int r = col->red();
	    int g = col->green();
	    int b = col->blue();
	    if ( mem.idx == Red )
		r = val.toInteger();
	    else if ( mem.idx == Green )
		g = val.toInteger();
	    else if ( mem.idx == Blue )
		b = val.toInteger();
	    col->setRgb( r, g, b );
	}
    } else {
	QSClass::fetchValue( objPtr, mem );
    }
}

QString QSColorClass::toString( const QSObject *obj ) const
{
    return color( obj )->name();
}

QString QSColorClass::debugString( const QSObject *obj ) const
{
    QColor *c = color( obj );
    return QString( "{name=%1:String,red=%2:Number,green=%3:Number,"
		    "blue=%4:Number}" ).arg( c->name() ).arg( c->red() )
	.arg( c->green() ).arg( c->blue() );
}

QSObject QSColorClass::construct( const QSList &args ) const
{
    if ( args.size() > 0 ) {
	if ( args.size() == 1 ) {
	    QSObject v( args[ 0 ] );
	    if ( v.isA( "Color" ) )
		return v;
	    else
		return construct( QColor( v.toString() ) );
	} else if ( args.size() == 3 ) {
	    QColor c( args[ 0 ].toInteger(),
		      args[ 1 ].toInteger(),
		      args[ 2 ].toInteger() );
	    return construct( c );
	}
    }

    return construct( QColor() );
}

QSObject QSColorClass::construct( const QColor &c ) const
{
    return QSObject( this, new ColorShared( c ) );
}

QSObject QSColorClass::setRgb( QSObject *objPtr, const QSList &args )
{
    assert( objPtr->isA( "Color" ) );
    QSColorClass *cc = (QSColorClass*)objPtr->objectType();
    QColor *col = cc->color( objPtr );

    if ( args.size() == 1 ) {
	col->setRgb( (uint)args[ 0 ].toInteger() );
    } else if ( args.size() == 3 ) {
	col->setRgb( (uint)args[ 0 ].toInteger(),
		     (uint)args[ 1 ].toInteger(),
		     (uint)args[ 2 ].toInteger() );
    }
    return QSUndefined();
}
