/****************************************************************************
** $Id: qsa.cpp  beta1   edited Dec 17 16:16 $
**
** Implementation of the QtApplicationScript class.
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsa.h"
#include "../kernel/quickinterpreter.h"
#include "../kernel/quickobjects.h"
#include "../kernel/quickdebugger.h"
#include "../engine/qsfuncref.h"
#include <qvariant.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qbuffer.h>
#include <private/qpluginmanager_p.h>
#include "qseditor.h"
#include "qsproject.h"
#include "qsinterpreter.h"
#include <stdlib.h>
#if defined(Q_OS_WIN32)
#include <qt_windows.h>
#endif

// designer
#include <mainwindow.h>
#include <sourcefile.h>
#include <formfile.h>
#include <formwindow.h>
#include <project.h>
#include <resource.h>
#include <hierarchyview.h>
#include <designerapp.h>
#include <../interfaces/projectsettingsiface.h>
#include <../interfaces/actioninterface.h>
#include <../interfaces/designerinterface.h>

#ifdef QSA_EXPIRES
static void qsa_eval_check();
#endif

class QSADesignerInterface : public DesignerInterface
{
public:
    QSADesignerInterface() {}

    DesignerProject *currentProject() const { return QtApplicationScript::self()->project->iFace(); }
    DesignerFormWindow *currentForm() const { return 0; }
    DesignerSourceFile *currentSourceFile() const { return 0; }
    QPtrList<DesignerProject> projectList() const {
	QPtrList<DesignerProject> l;
	l.append( QtApplicationScript::self()->project->iFace() );
	return l;
    }
    void showStatusMessage( const QString &, int ) const {}
    DesignerDock *createDock() const { return 0; }
    DesignerOutputDock *outputDock() const { return 0; }
    void setModified( bool, QWidget * ) {}
    void updateFunctionList() {}

    void onProjectChange( QObject *, const char * ) {}
    void onFormChange( QObject *, const char * ) {}

    bool singleProjectMode() const { return TRUE; }

    QRESULT queryInterface( const QUuid &uuid, QUnknownInterface** iface ) {
	*iface = 0;

	if ( uuid == IID_QUnknown )
	    *iface = (QUnknownInterface*)this;
	else if ( uuid == IID_QComponentInformation )
	    *iface = (QComponentInformationInterface*)this;
	else if ( uuid == IID_Designer )
	    *iface = (DesignerInterface*)this;
	else
	    return QE_NOINTERFACE;

	(*iface)->addRef();
	return QS_OK;
    }
    Q_REFCOUNT

};



// declared in QWidgetFactory
extern QString *qwf_plugin_dir;

QtApplicationScript *QtApplicationScript::selfPtr = 0;
static QPluginManager<ActionInterface> *actionPluginManager = 0;
static QPluginManager<ProjectSettingsInterface> *projectSettingsPluginManager = 0;

void deleteSelfPtr()
{
    delete QtApplicationScript::selfPtr;
    QtApplicationScript::selfPtr = 0;
}

/*!
  \internal
*/

QtApplicationScript::QtApplicationScript()
    : designer( 0 ), projectOpened( FALSE ),
      scriptRunning( FALSE ), projectRunning( FALSE ),
      em( AskForDebug ), blockErrors( FALSE ), project( 0 ),
      needRerun( FALSE )
{
#ifdef QSA_EXPIRES
    if ( !selfPtr ) {
	qsa_eval_check();
    }
#endif

    if ( !qwf_plugin_dir )
	qwf_plugin_dir = new QString( "/qsa" );
    else
	*qwf_plugin_dir = QString( "/qsa" );
    setupPluginManagers();
    signalHandlers = new QValueList<QSSignalHandler>;
    listEditors = new QPtrList<QSEditor>;
    DesignerApplication::setSettingsKey( "/QSA Developer/1.0/" );
    appObjs.setAutoDelete( FALSE );
    if ( selfPtr ) {
	qFatal( "QtApplicationScript::QtApplicationScript: Only one instance of QtApplicationScript "
		"may exist at any one time!" );
    }
    selfPtr = this;
    connect( QuickInterpreter::self(), SIGNAL( runtimeError() ),
	     this, SLOT( runtimeError() ) );
    qsip = 0;
    prj = new QSProject( this );
}

/*! Destructor
  \internal
*/

QtApplicationScript::~QtApplicationScript()
{
    delete signalHandlers;
    signalHandlers = 0;
    delete listEditors;
    listEditors = 0;
    if ( project )
	project->removeTempProject();

    if ( designer ) {
	designer->writeConfig();
	designer->shuttingDown();
    } else if ( project ) {
	delete project;
    }
    delete prj;
    if ( selfPtr == this )
	selfPtr = 0;
}


QSProject* QtApplicationScript::currentProject() const
{
    return prj;
}

QSInterpreter* QtApplicationScript::interpreter() const
{
    if ( !qsip )
	((QtApplicationScript*)this)->qsip = new QSInterpreter();

    return qsip;
}

/*! \internal */

QtApplicationScript *QtApplicationScript::self()
{
    if ( !selfPtr ) {
	(void)new QtApplicationScript;
	qAddPostRoutine( deleteSelfPtr );
    }
    return selfPtr;
}

/*! \internal */

void QtApplicationScript::addObject( QObject *o )
{
    if ( !o )
	return;
#if defined(QT_CHECK_RANGE)
    if ( strcmp( o->name(), "unnamed" ) == 0 )
	qWarning( "QtApplicationScript::addObject: unnamed object" );
#endif
    QuickInterpreter::self()->addTopLevelObject( o );
    appObjs.append( o );
    connect( o, SIGNAL( destroyed( QObject * ) ), this, SLOT( objectDestroyed( QObject * ) ) );
    if ( project )
	project->addObject( o );
}

/*! \internal */

void QtApplicationScript::removeObject( QObject *o )
{
    if ( appObjs.removeRef( o ) && project )
	project->removeObject( o );
    QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
    while ( it != signalHandlers->end() ) {
	QValueList<QSSignalHandler>::Iterator it2 = it;
	QSSignalHandler sigh = *it2;
	++it;
	if ( sigh.sender == o || sigh.receiver == o )
	    signalHandlers->remove( it2 );
    }
}

/*! \internal */

const QObjectList* QtApplicationScript::objects() const
{
    return &appObjs;
}

/*! \internal */

QStringList QtApplicationScript::scriptFiles() const
{
    if ( !project )
	return QStringList();
    QPtrListIterator<SourceFile> it( project->sourceFiles() );
    QStringList lst;
    while ( it.current() ) {
	lst << it.current()->fileName();
	++it;
    }
    return lst;
}

/*! \internal */

QString QtApplicationScript::objectSource( QObject *object ) const
{
    if ( !project )
	return QString::null;
    FormFile *ff = project->fakeFormFileFor( object );
    if ( !ff )
	return QString::null;
    return ff->code();
}

/*! \internal */

QString QtApplicationScript::formXML( const QString &formName ) const
{
    if ( !project )
	return QString::null;
    QPtrListIterator<FormFile> it = project->formFiles();
    FormFile *ff = 0;
    while ( it.current() ) {
	if ( it.current()->isFake() ) {
	    ++it;
	    continue;
	}
	if ( QString( it.current()->formName() ) == formName ) {
	    ff = it.current();
	    break;
	}
	++it;
    }

    if ( !ff )
	return QString::null;

    if ( !ff->formWindow() ) {
	QFile f( ff->fileName() );
	if ( !f.open( IO_ReadOnly ) )
	    return QString::null;

	QTextStream ts( &f );
	return ts.read();
    }

    QCString s;
    QBuffer buffer( s );
    buffer.open( IO_WriteOnly );
    Resource resource( designer );
    resource.setWidget( ff->formWindow() );
    resource.save( &buffer );
    return QString( s );
}

/*! \internal */

QString QtApplicationScript::formSource( const QString &formName ) const
{
    if ( !project )
	return QString::null;
    QPtrListIterator<FormFile> it = project->formFiles();
    FormFile *ff = 0;
    while ( it.current() ) {
	if ( it.current()->formName() == formName ) {
	    ff = it.current();
	    break;
	}
	++it;
    }

    if ( !ff )
	return QString::null;
    return ff->code();
}

/*! \internal */

QString QtApplicationScript::source( const QString &fileName ) const
{
    if ( !project )
	return QString::null;
    SourceFile *sf = project->findSourceFile( fileName );
    if ( !sf )
	return QString::null;
    return sf->text();
}

/*! \internal */

void QtApplicationScript::clearObjects()
{
    if ( project ) {
	for ( QObject *o = appObjs.first(); o; o = appObjs.next() )
	    project->removeObject( o );
    }
    appObjs.clear();
}

/*! \internal */

void QtApplicationScript::setObjectSource( QObject *object, const QString &code, bool append )
{
    if ( !project )
	return;
    int sourceId;
    QuickInterpreter::self()->execute( object, code, sourceId );
    QuickInterpreter::self()->addSourceId( sourceId, object );
    FormFile *ff = project->fakeFormFileFor( object );
    if ( !ff )
	return;
    QString cd( code );
    if ( append && !ff->code().isEmpty() )
	cd = ff->code() + "\n" + cd;
    ff->parseCode( cd, TRUE );
    ff->setCode( cd );
    ff->setModified( TRUE );
    project->setModified( TRUE );
    if ( ff->editor() )
	ff->editor()->editorInterface()->setText( cd );
}


/*! \internal */

void QtApplicationScript::addSource( const QString &code, const QString &fileName, bool append )
{
    if ( !project )
	return;
    int sourceId;
    QuickInterpreter::self()->execute( 0, code, sourceId );
    SourceFile *sf = project->findSourceFile( project->makeRelative( fileName ) );
    if ( !sf )
	sf = new SourceFile( project->makeRelative( fileName ), FALSE, project );
    QString cd( code );
    if ( append )
	cd = sf->text() + "\n" + code;
    project->setModified( TRUE );
    sf->setText( cd );
    sf->setModified( TRUE );
    if ( !sf->isModified() || // this is a hack, look at SourceFile::isModified() for an explaination
	 !QFile::exists( project->makeAbsolute( fileName ) ) ) {
	QFile fl( project->makeAbsolute( fileName ) );
	if ( fl.open( IO_WriteOnly ) ) {
	    QTextStream ts( &fl );
	    ts << cd;
	}
    }
}

/*! \internal */

void QtApplicationScript::removeSource( const QString &fileName )
{
    if ( !project )
	return;
    SourceFile *sf = project->findSourceFile( project->makeRelative( fileName ) );
    if ( sf )
	project->removeSourceFile( sf );
}

/*! \internal */

void QtApplicationScript::addForm( const QString &fileName )
{
    if ( !project )
	return;
    Resource resource( designer );
    FormFile *ff = new FormFile( project->makeRelative( fileName ),
				 FALSE, project );
    resource.load( ff, project );
}

/*! \internal */

void QtApplicationScript::removeForm( const QString &fileName )
{
    if ( !project )
	return;
    FormFile *ff = project->findFormFile( project->makeRelative( fileName ) );
    if ( ff )
	project->removeFormFile( ff );
}

/*! \internal */

void QtApplicationScript::addSignalHandler( QObject *sender, const char *signal,
				    QObject *receiver, const char *qtscriptFunction )
{
    checkProject();
    QuickInterpreter *ip = QuickInterpreter::self();
    QString func( qtscriptFunction );
    QSObject senderObj = ip->wrap( sender );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->setEventHandler( signal + 1, receiver, func.left( func.find( '(' ) ) );

    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = receiver;
    sigh.signal = signal + 1;
    sigh.function = func.left( func.find( '(' ) );
    (*signalHandlers) << sigh;

    connect( sender, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
    connect( receiver, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
}

/*! \internal */

void QtApplicationScript::addSignalHandler( QObject *sender, const char *signal,
					    const char *qtscriptFunction )
{
    checkProject();
    QuickInterpreter *ip = QuickInterpreter::self();
    QString func( qtscriptFunction );
    func.left( func.find( '(' ) );
    QSObject senderObj = ip->wrap( sender );

    QSObject obj = ip->object( func );
    QSObject base = QSFuncRefClass::refBase( obj );
    QSMember member = QSFuncRefClass::refMember( obj );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->setEventHandler( signal + 1, 0, member.name(), base );

    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = 0;
    sigh.signal = signal + 1;
    sigh.function = func;
    (*signalHandlers) << sigh;

    connect( sender, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
}

/*! \internal */

void QtApplicationScript::removeSignalHandler( QObject *sender, const char *signal,
					       QObject *receiver, const char *qtscriptFunction )
{
    checkProject();
    QString func( qtscriptFunction );
    QuickInterpreter *ip = QuickInterpreter::self();
    QSObject senderObj = ip->wrap( sender );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->removeEventHandler( signal + 1, receiver, func.left( func.find( '(' ) ) );
    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = receiver;
    sigh.signal = signal + 1;
    sigh.function = func.left( func.find( '(' ) );
    for ( QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
	  it != signalHandlers->end(); ++it ) {
	if ( sigh == *it ) {
	    signalHandlers->remove( it );
	    break;
	}
    }
}

/*! \internal */

void QtApplicationScript::removeSignalHandler( QObject *sender, const char *signal,
					       const char *qtscriptFunction )
{
    checkProject();

    QuickInterpreter *ip = QuickInterpreter::self();
    QString func( qtscriptFunction );
    func.left( func.find( '(' ) );
    QSObject senderObj = ip->wrap( sender );

    QSObject obj = ip->object( func );
    QSObject base = QSFuncRefClass::refBase( obj );
    QSMember member = QSFuncRefClass::refMember( obj );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->removeEventHandler( signal + 1, 0, member.name(), base );

    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = 0;
    sigh.signal = signal + 1;
    sigh.function = func;
    for ( QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
	  it != signalHandlers->end(); ++it ) {
	if ( sigh == *it ) {
	    signalHandlers->remove( it );
	    break;
	}
    }
}

/*! \internal */

QVariant QtApplicationScript::evaluate( const QString &code, QObject *context )
{
    if ( scriptRunning ) {
	qWarning( "QtApplicationScript::evaluate: cannot recursively call"
		  " evaluate(), call() or checkSyntax()" );
	return QVariant();
    }
    scriptRunning = TRUE;
    checkProject();
    int id;
    QVariant v = QuickInterpreter::self()->execute( context, code, id );
    stopProject();
    scriptRunning = FALSE;
    return v;
}

/*! \internal */

QVariant QtApplicationScript::call( const QString &function,
				    const QValueList<QVariant> &arguments, QObject *context )
{
    if ( scriptRunning ) {
	qWarning( "QtApplicationScript::call: cannot recursively call"
		  " call(), evaluate() or checkSyntax()" );
	return QVariant();
    }
    scriptRunning = TRUE;
    checkProject();
    QVariant v = QuickInterpreter::self()->call( context, function, arguments );
    stopProject();
    scriptRunning = FALSE;
    return v;
}

/*! \internal */

bool QtApplicationScript::checkSyntax( const QString &code )
{
    if ( scriptRunning ) {
	// ### check whether this limitation still exists
	qWarning( "QtApplicationScript::checkSyntax: cannot recursively call"
		  " call(), evaluate() or checkSyntax()" );
	return FALSE;
    }
    scriptRunning = TRUE;
    checkProject();
    bool ok = QuickInterpreter::self()->checkSyntax( code );
    stopProject();
    scriptRunning = FALSE;
    return ok;
}

/*! \internal */

void QtApplicationScript::open( const QByteArray &projectData, const QString &projectName )
{
    // ### don't hardcode tmp dir
    QString qsa = QString( getenv( "HOME" ) ) + QString( "/.qsa" );
    if ( !QFile::exists( qsa ) ) {
	QDir d;
	d.mkdir( qsa );
    }

    // ### avoid collisions
    QString fn = QString( getenv( "HOME" ) ) + QString( "/.qsa/tmp_" ) + projectName + QString( ".qsa" );
    if ( QFile::exists( fn ) ) {
	QDir d;
	d.remove( fn );
    }
    QDir d;
    d.mkdir( QString( getenv( "HOME" ) ) + QString( "/.qsa/tmp_tmp_" ) + projectName );
    if ( projectData.size() > 0 ) {
	QFile f( fn );
	if ( f.open( IO_WriteOnly ) ) {
	    f.writeBlock( projectData );
	    f.close();
	}
    }
    open( fn );
}

/*! \internal */

void QtApplicationScript::open( const QString &scriptProject )
{
    clear();

    QString qsa = QString( getenv( "HOME" ) ) + QString( "/.qsa" );
    if ( !QFile::exists( qsa ) ) {
	QDir d;
	d.mkdir( qsa );
    }

    setupPluginManagers();

    project = new Project( QFileInfo( scriptProject ).absFilePath(), "",
			   projectSettingsPluginManager, FALSE, "Qt Script" );

    if ( !QFile::exists( project->makeAbsolute( project->fileName() ) ) ) {
	SourceFile *f = new SourceFile( "main.qs", FALSE, project );
	f->setText( "// Put all global functions here\n" );
	f->setModified( TRUE );
	if ( designer )
	    designer->editSource( f );
    }

    if ( designer )
	designer->setSingleProject( project );


    project->setObjects( appObjs );
    connect( project, SIGNAL( runtimeError( const QString &, const QString &, int ) ),
	     this, SLOT( runtimeError( const QString &, const QString &, int ) ) );
    connect( project, SIGNAL( runFinished() ),
	     this, SLOT( runFinished() ) );
    connect( project, SIGNAL( newFormOpened( FormWindow * ) ),
	     this, SLOT( formAdded( FormWindow * ) ) );

    projectLastModified = QFileInfo( scriptProject ).lastModified();
    projectName = scriptProject;
    projectOpened = TRUE;

    checkProject();
}

/*! \internal */

QByteArray QtApplicationScript::projectData() const
{
    QByteArray ba;
    if ( !projectOpened )
	return ba;
    project->save();
    QFile f( project->fileName( TRUE ) );
    f.open( IO_ReadOnly );
    ba.resize( f.size() );
    f.readBlock( ba.data(), ba.size() );
    f.close();
    return ba;
}

/*! \internal */

void QtApplicationScript::clear()
{
    if ( project )
	project->save();
    closeDeveloper();
    if ( projectOpened )
	QuickInterpreter::self()->emitStopProject();
    projectOpened = FALSE;
    QuickInterpreter::self()->clear();
    scriptRunning = FALSE;
    projectRunning = FALSE;
    if ( project )
	project->removeTempProject();
}

/*! \internal */

void QtApplicationScript::openDeveloper( int flags )
{
    if ( !projectOpened )
	return;

    setupDesigner();

    designer->showGUIStuff( ( flags & GUIBuilder )== GUIBuilder );
    designer->setEditorsReadOnly( ( flags & EditableEditor ) != EditableEditor && !!activeEditor() );
    designer->toggleSignalHandlers( ( flags & SignalHandlerTab ) == SignalHandlerTab );
    QuickInterpreter::self()->emitToggleDebugger( ( flags & Debugger ) == Debugger );

    designer->show();
    designer->raise();
    designer->removeEventFilter( this );
    designer->installEventFilter( this );

    stopProject( TRUE );
}

/*! \internal */

void QtApplicationScript::closeDeveloper()
{
    if ( !projectOpened )
	return;

    if ( designer )
	designer->hide();
}

/*! \internal */

bool QtApplicationScript::addFunction( const QString &functionName, const QString &functionBody,
				       bool openDeveloper )
{
    if ( !projectOpened )
	return FALSE;

    static QString letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUFWXYZ_";
    static QString numbers = "0123456789";
    for ( int i = 0; i < (int)functionName.length(); ++i ) {
	if ( i == 0 && letters.find( functionName[i] ) == -1 ||
	     letters.find( functionName[i] ) == -1 && numbers.find( functionName[i] ) == -1 ) {
#if defined(QT_CHECK_RANGE)
	    qWarning( "QtApplicationScript::addFunction: '%s' is not a valid function name",
		      functionName.latin1() );
#endif
	    return FALSE;
	}
    }

    if ( openDeveloper )
	QtApplicationScript::openDeveloper();
    project->addAndEditFunction( functionName, functionBody, openDeveloper );
    return TRUE;
}

/*! \internal */

QStringList QtApplicationScript::globalFunctions( bool includeSignature ) const
{
    return functionsOf( QString::null, includeSignature );
}

/*! \internal */

QStringList QtApplicationScript::functionsOf( const QString &context, bool includeSignature ) const
{
    checkAndStopProject();
    QSObject obj = QuickInterpreter::self()->object( context );
    return QuickInterpreter::self()->functionsOf( obj, includeSignature );
}

/*! \internal */

QStringList QtApplicationScript::functionsOf( QObject *context, bool includeSignature ) const
{
    checkAndStopProject();
    QSObject obj = QuickInterpreter::self()->wrap( context );
    if ( obj.isUndefined() )
	return QStringList();
    return QuickInterpreter::self()->functionsOf( obj, includeSignature );
}

/*! \internal */

QStringList QtApplicationScript::classes() const
{
    checkAndStopProject();
    return QuickInterpreter::self()->classes();
}

/*! \internal */

QStringList QtApplicationScript::classesOf( const QString &context ) const
{
    checkAndStopProject();
    QSObject obj = QuickInterpreter::self()->object( context );
    return QuickInterpreter::self()->classesOf( obj );
}

/*! \internal */

QStringList QtApplicationScript::classesOf( QObject *context ) const
{
    checkAndStopProject();
    QSObject obj = QuickInterpreter::self()->wrap( context );
    if ( obj.isUndefined() )
	return QStringList();
    return QuickInterpreter::self()->classesOf( obj );
}

/*! \internal */

QStringList QtApplicationScript::globalClasses() const
{
    return classesOf( QString::null );
}

/*! \internal */

QStringList QtApplicationScript::variablesOf( const QString &context ) const
{
    checkAndStopProject();
    QSObject obj = QuickInterpreter::self()->object( context );
    return QuickInterpreter::self()->variablesOf( obj );
}

/*! \internal */

QStringList QtApplicationScript::variablesOf( QObject *context ) const
{
    checkAndStopProject();
    QSObject obj = QuickInterpreter::self()->wrap( context );
    if ( obj.isUndefined() )
	return QStringList();
    return QuickInterpreter::self()->variablesOf( obj );
}

/*! \internal */

QStringList QtApplicationScript::globalVariables() const
{
    return variablesOf( QString::null );
}

/*! \internal */

QStringList QtApplicationScript::forms() const
{
    if ( !project )
	return QStringList();

    QPtrListIterator<FormFile> it = project->formFiles();
    QStringList lst;
    while ( it.current() ) {
	if ( it.current()->isFake() ) {
	    ++it;
	    continue;
	}
	lst << it.current()->formName();
	++it;
    }
    return lst;
}

/*! \internal */

void QtApplicationScript::reset()
{
    if ( !projectOpened )
	return;
    checkProject();
}

/*! This function is called when a new QSEditor has been
  created.

  \internal
*/

void QtApplicationScript::registerEditor( QSEditor *editor )
{
    listEditors->append( editor );
}

/*! This function is called when a QSEditor has been
  detroyed.

  \internal
*/

void QtApplicationScript::unregisterEditor( QSEditor *editor )
{
    listEditors->removeRef( editor );
}

/*! \internal */

const QPtrList<QSEditor> *QtApplicationScript::editors() const
{
    return (const QPtrList<QSEditor>*)listEditors;
}

/*! \internal */

QSEditor *QtApplicationScript::activeEditor() const
{
    for ( QSEditor *e = listEditors->first(); e; e = listEditors->next() ) {
	if ( e->isActive() )
	    return e;
    }
    return 0;
}

/*! \internal */

QObject *QtApplicationScript::currentContext() const
{
    QSObject o = QuickInterpreter::self()->env()->currentScope();
    if ( o.isA( QuickInterpreter::self()->wrapperClass() ) )
	return QuickInterpreter::self()->wrapperClass()->shared( &o )->objects[0];
    return 0;
}

void QtApplicationScript::objectDestroyed( QObject *o )
{
    if ( o == designer )
	designer = 0;
    else
	removeObject( o );
}

bool QtApplicationScript::checkProject()
{
    QuickInterpreter *i = QuickInterpreter::self();
    int iid = i->uniqueId();
    if ( projectOpened && project ) {
	hadError = FALSE;
	if ( !projectRunning ) {
	    QuickInterpreter::self()->emitRunProject();
	    projectLastModified = QFileInfo( projectName ).lastModified();
	} else if ( project->isModified() ) {
	    project->save();
	    QuickInterpreter::self()->emitStopProject();
	    QuickInterpreter::self()->emitRunProject();
	    projectLastModified = QFileInfo( projectName ).lastModified();
	} else if ( QFileInfo( projectName ).lastModified() != projectLastModified ) {
	    QuickInterpreter::self()->emitStopProject();
	    QuickInterpreter::self()->emitRunProject();
	    projectLastModified = QFileInfo( projectName ).lastModified();
	} else if ( needRerun ) {
	    QuickInterpreter::self()->emitStopProject();
	    QuickInterpreter::self()->emitRunProject();
	}
	needRerun = FALSE;
	projectRunning = !hadError;
	hadError = FALSE;
    }
    if ( iid != QuickInterpreter::self()->uniqueId() ) {
	connect( QuickInterpreter::self(), SIGNAL( runtimeError() ),
		 this, SLOT( runtimeError() ) );
	readdEventHandlers();
    }

    return iid == QuickInterpreter::self()->uniqueId();
}

void QtApplicationScript::runtimeError()
{
    if ( blockErrors || project )
	return;
    int l = QuickInterpreter::self()->debuggerEngine()->lineNumber();
    if ( l < 0 )
	l = QuickInterpreter::self()->errorLines().first();
    else
	l++;
    runtimeError( QuickInterpreter::self()->errorMessages().first(),
		  "<unknown>",
		  l );
}

void QtApplicationScript::runtimeError( const QString &message, const QString &filename, int lineNumber )
{
    if ( blockErrors )
	return;

    hadError = TRUE;
    emit error( message, filename, lineNumber );

    if ( !projectOpened && qApp->type() != QApplication::Tty ) {
	if ( em != Nothing ) {
	    QMessageBox::critical( qApp->mainWidget(), "Error", "The following error occurred in "
				   "line <b>" + QString::number( lineNumber ) +
				   "</b> of file <b>" + filename +
				   "</b> while executing "
				   "the script:<pre><font color=red>" +
				   message + "</font></pre>" );
	}
	return;
    }

    QuickInterpreter::self()->emitStopProject();
    projectRunning = FALSE;

    if ( qApp->type() == QApplication::Tty )
	return;

    bool debug = FALSE;
    if ( !designer || !designer->isVisible() ) {
	switch ( em ) {
	case Notify:
	    QMessageBox::critical( qApp->mainWidget(), "Error", "The following error occurred in "
				   "line <b>" + QString::number( lineNumber ) +
				   "</b> of file <b>" + filename +
				   "</b> while executing "
				   "the script:<pre><font color=red>" +
				   message + "</font></pre>" );
	    break;
	case AskForDebug:
	    debug = QMessageBox::critical( qApp->mainWidget(), "Error", "The following error occurred in "
					   "line <b>" + QString::number( lineNumber ) +
					   "</b> of file <b>" + filename +
					   "</b> while executing "
					   "the script:<pre><font color=red>" +
					   message + "</font></pre>"
					   "Do you want to debug the script?", "&Yes", "&No",
					   "", 0, 1 ) == 0;
	    break;
	case Debug:
	    debug = TRUE;
	    break;
	case Nothing:
	    break;
	}
    } else {
	debug = TRUE;
    }

    if ( debug ) {
	if ( !designer )
	    setupDesigner();
	designer->show();
	designer->raise();
    }
}

void QtApplicationScript::stopProject( bool force )
{
    if ( !projectOpened )
	return;
    if ( !force && ( !designer || !designer->isVisible() ) && !activeEditor() )
	return;
    QuickInterpreter::self()->emitStopProject();
    readdObjects();
    projectRunning = FALSE;
}

void QtApplicationScript::runFinished()
{
    projectRunning = FALSE;
    scriptRunning = FALSE;
}

/*! \reimp */

bool QtApplicationScript::eventFilter( QObject *o, QEvent *e )
{
    if ( o == designer && e->type() == QEvent::Hide ) {
	blockErrors = TRUE;
	// the hide event has to be processed first
 	QTimer::singleShot( 0, this, SLOT( checkProject() ) );
	blockErrors = FALSE;
    } else if ( o == designer && e->type() == QEvent::Show ) {
 	stopProject( TRUE );
    }

    return QObject::eventFilter( o, e );
}

void QtApplicationScript::setupPluginManagers()
{
    if ( actionPluginManager )
	return;

    MetaDataBase::setupInterfaceManagers( "/qsa" );
    projectSettingsPluginManager =
	new QPluginManager<ProjectSettingsInterface>( IID_ProjectSettings,
						      QApplication::libraryPaths(),
						      "/qsa" );

    // That part is hacky. To instantiate the debugger in QuickIDE, we
    // have to load the action plugins. This should be solved properly
    // by having a DebuggerInterface or so, as the way we do it is
    // just too hacky

    actionPluginManager = new QPluginManager<ActionInterface>( IID_Action,
						       QApplication::libraryPaths(),
						       "/qsa" );

    QStringList lst = actionPluginManager->featureList();
    QSADesignerInterface *dIface = new QSADesignerInterface;
    for ( QStringList::Iterator ait = lst.begin(); ait != lst.end(); ++ait ) {
	ActionInterface *iface = 0;
	actionPluginManager->queryInterface( *ait, &iface );
	if ( !iface )
	    continue;
	if ( !(*ait).startsWith( "Qt Script Debugger" ) )
	    continue;
	iface->connectTo( dIface );
	iface->release();
    }
}

void QtApplicationScript::setupDesigner()
{
    if ( designer )
	return;
    QApplication::setOverrideCursor( waitCursor );
    designer = new MainWindow( FALSE, TRUE, "/qsa" );
    designer->setCaption( tr( "Qt Script for Applications Developer" ) );
    connect( designer, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
    QApplication::restoreOverrideCursor();

    designer->setSingleProject( project );
}

void QtApplicationScript::readdObjects()
{
    QuickInterpreter *ip = QuickInterpreter::self();
    for ( QObject *o = appObjs.first(); o; o = appObjs.next() )
	ip->addTopLevelObject( o );
    if ( project ) {
	QPtrListIterator<FormFile> it = project->formFiles();
	while ( it.current() ) {
	    if ( !it.current()->isFake() && it.current()->formWindow() )
		ip->addTopLevelObject( it.current()->formWindow()->mainContainer() );
	    ++it;
	}
    }
}

void QtApplicationScript::readdEventHandlers()
{
    QuickInterpreter *ip = QuickInterpreter::self();
    for ( QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
	  it != signalHandlers->end(); ++it ) {
	QSSignalHandler sigh = *it;
	QSObject senderObj = ip->wrap( sigh.sender );
	QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
	if ( sigh.receiver ) {
	    sh->setEventHandler( sigh.signal, sigh.receiver, sigh.function );
	} else {
	    QSObject obj = ip->object( sigh.function );
	    QSObject base = QSFuncRefClass::refBase( obj );
	    QSMember member = QSFuncRefClass::refMember( obj );
	    sh->setEventHandler( sigh.signal, 0, member.name(), base );
	}
    }
}

void QtApplicationScript::formAdded( FormWindow *fw )
{
    QuickInterpreter *ip = QuickInterpreter::self();
    ip->addTopLevelObject( fw->mainContainer() );
}

void QtApplicationScript::checkAndStopProject() const
{
    ( (QtApplicationScript*)this)->blockErrors = TRUE;
    ( (QtApplicationScript*)this)->checkProject();
    ( (QtApplicationScript*)this)->stopProject();
    ( (QtApplicationScript*)this)->blockErrors = FALSE;
}

#ifdef QSA_EXPIRES
static void qsa_eval_check()
{
    QDate date( QSA_EXPIRES>>16, (QSA_EXPIRES&0xff00)>>8, (QSA_EXPIRES&0xff) );
    if ( date > QDate::currentDate() )
	return;
    QMessageBox::warning( 0,
			  QObject::tr( "End of Evaluation Period" ),
			  QObject::tr( "The evaluation period of QSA has expired.\n\n"
				       "Please check http://www.trolltech.com/products/qsa for updates\n"
				       "or contact sales@trolltech.com for further information" ),
			  QMessageBox::Ok,
			  QMessageBox::NoButton );
}
#endif
