/****************************************************************************
** $Id: qsproject.h  beta1   edited Dec 17 09:47 $
**
** Definition of the QSProject class.
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QSPROJECT_H
#define QSPROJECT_H

#include <qobject.h>
#include "qsaglobal.h"

class QtApplicationScript;
class QSProjectPrivate;

class QSA_EXPORT QSProject : public QObject
{
    friend class QtApplicationScript;
    Q_OBJECT

public:
    enum DeveloperFlags {
	GUIBuilder = 1,
	Debugger = 2,
	SignalHandlerTab = 4,
	EditableEditor = 8,
	FullDeveloper = GUIBuilder | Debugger | SignalHandlerTab | EditableEditor,
	OnlyEditor = EditableEditor | SignalHandlerTab,
	OnlySimpleEditor = EditableEditor,
	OnlyEditorAndDebugger = EditableEditor | Debugger | SignalHandlerTab
    };

    QStringList forms() const;
    QStringList scriptFiles() const;
    QString formSource( const QString &formName ) const;
    QString formXML( const QString &formName ) const;
    QString source( const QString &fileName ) const;
    QByteArray projectData() const;

public slots:
    void open( const QString &scriptProject );
    void open( const QByteArray &projectData, const QString &projectName );

    void addSource( const QString &code, const QString &fileName,
		    bool append = FALSE );
    void removeSource( const QString &fileName );
    void addForm( const QString &fileName );
    void removeForm( const QString &fileName );

    void openDeveloper( int flags = FullDeveloper );
    void closeDeveloper();
    bool addFunction( const QString &functionName,
		      const QString &functionBody = QString::null,
		      bool openDeveloper = TRUE );

    void reset();

private:
    QSProject( QtApplicationScript *s );
    virtual ~QSProject();
    // disabled copy constructor and assignment operator
    QSProject( const QSProject & );
    QSProject& operator=( const QSProject & );


private:
    QtApplicationScript *qas;
    QSProjectPrivate *d;

};

#endif
